@@ -13,6 +13,8 @@
  * CS 42112, 75589 Paris Cedex 12, France
  */
 
+#include <isl/val.h>
+#include <isl/space.h>
 #include <isl/map.h>
 #include <isl_schedule_band.h>
 #include <isl_schedule_private.h>
@@ -82,7 +84,7 @@ __isl_take isl_schedule_tree *isl_schedule_tree_dup(
 	case isl_schedule_node_error:
 		isl_die(ctx, isl_error_internal,
 			"allocation should have failed",
-			isl_schedule_tree_free(dup));
+			return isl_schedule_tree_free(dup));
 	case isl_schedule_node_band:
 		dup->band = isl_schedule_band_copy(tree->band);
 		if (!dup->band)
@@ -1731,7 +1733,7 @@ static __isl_give isl_union_map *subtree_schedule_extend_from_children(
 	separate = n > 1 && (tree->type == isl_schedule_node_sequence ||
 			    isl_options_get_schedule_separate_components(ctx));
 
-	space = extract_space_from_filter_child(tree);
+	space = isl_space_params_alloc(ctx, 0);
 
 	umap = isl_union_map_empty(isl_space_copy(space));
 	space = isl_space_set_from_params(space);
@@ -1743,6 +1745,7 @@ static __isl_give isl_union_map *subtree_schedule_extend_from_children(
 
 	dim = isl_multi_val_dim(mv, isl_dim_set);
 	for (i = 0; i < n; ++i) {
+		isl_multi_val *mv_copy;
 		isl_union_pw_multi_aff *upma;
 		isl_union_map *umap_i;
 		isl_union_set *dom;
@@ -1758,8 +1761,10 @@ static __isl_give isl_union_map *subtree_schedule_extend_from_children(
 			mv = isl_multi_val_set_val(mv, 0, isl_val_copy(v));
 			v = isl_val_add_ui(v, 1);
 		}
-		upma = isl_union_pw_multi_aff_multi_val_on_domain(dom,
-							isl_multi_val_copy(mv));
+		mv_copy = isl_multi_val_copy(mv);
+		space = isl_union_set_get_space(dom);
+		mv_copy = isl_multi_val_align_params(mv_copy, space);
+		upma = isl_union_pw_multi_aff_multi_val_on_domain(dom, mv_copy);
 		umap_i = isl_union_map_from_union_pw_multi_aff(upma);
 		umap_i = isl_union_map_flat_range_product(
 					    isl_union_map_copy(outer), umap_i);
@@ -2265,7 +2270,7 @@ static __isl_give isl_set *isolate_final(__isl_keep isl_set *isolate,
  * The tree is itself positioned at schedule depth "depth".
  *
  * The loop AST generation type options and the isolate option
- * are split over the the two band nodes.
+ * are split over the two band nodes.
  */
 __isl_give isl_schedule_tree *isl_schedule_tree_band_split(
 	__isl_take isl_schedule_tree *tree, int pos, int depth)
